# -*- coding: utf-8 -*-

from imio.smartweb.common.interfaces import ICropping
from operator import itemgetter
from plone.app.imagecropping.browser.editor import CroppingEditor
from plone.app.imagecropping.browser.settings import ISettings
from plone.app.imagecropping.dx import CroppingImageScalingFactory
from plone.app.imagecropping.storage import Storage
from plone.namedfile.interfaces import IAvailableSizes
from plone.registry.interfaces import IRegistry
from zope.component import getUtility

import six


class SmartwebCroppingEditor(CroppingEditor):
    def _scale_info(self, fieldname, scale_id, target_size, true_size):
        scale = super(SmartwebCroppingEditor, self)._scale_info(
            fieldname, scale_id, target_size, true_size
        )
        title = scale["title"]
        if "portrait" in title or "paysage" in title or "carre" in title:
            # remove orientation part from scale title
            title = title.split("_")[0]
        scale["title"] = title.title()
        return scale

    def _scales(self, fieldname):
        adapter = ICropping(self.context, alternate=None)
        if adapter is None:
            yield from super(SmartwebCroppingEditor, self)._scales(fieldname)
            return
        allowed_sizes = getUtility(IAvailableSizes)() or []
        sizes_iterator = sorted(six.iteritems(allowed_sizes), key=itemgetter(1))
        context_scales = adapter.get_scales(fieldname, self.request)
        for scale_id, target_size in sizes_iterator:
            if scale_id not in context_scales:
                continue
            yield scale_id, target_size


class SmartwebCroppingImageScalingFactory(CroppingImageScalingFactory):
    def __call__(
        self,
        fieldname=None,
        mode="scale",
        height=None,
        width=None,
        scale=None,
        **parameters,
    ):
        storage = Storage(self.context)
        self.box = storage.read(fieldname, scale)
        if scale is not None:
            if "portrait" in scale or "paysage" in scale or "carre" in scale:
                orientation = scale.split("_")[0]
                # take cropping box from "affiche" scale
                self.box = storage.read(fieldname, f"{orientation}_affiche")
        if self.box:
            mode = "contain"
        else:
            registry = getUtility(IRegistry)
            settings = registry.forInterface(ISettings)
            if scale in settings.cropping_for:
                mode = "contain"
        return super(CroppingImageScalingFactory, self).__call__(
            fieldname=fieldname,
            mode=mode,
            height=height,
            width=width,
            scale=scale,
            **parameters,
        )



# -*- coding: utf-8 -*-

from collective.taxonomy.interfaces import ITaxonomy
from collective.taxonomy.jsonimpl import EditTaxonomyData as baseEditTaxonomyData
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from zope.component import queryUtility
from zope.component import queryUtility
from plone import api
from Products.Five import BrowserView


import json


class EditTaxonomyData(baseEditTaxonomyData, BrowserView):
    """Edit taxonomy data."""


class DeleteTaxonomyData(BrowserView):

    def check_delete_taxonomy(self):
        body = self.request.get("BODY", b"{}")  # Récupération du corps de la requête
        data = json.loads(body)  # Décodage du JSON
        term_id = data.get("termId")  # Extraction de l'ID
        result = {term_id} if term_id else set()
        taxonomy = queryUtility(ITaxonomy, name="collective.taxonomy.contact_category")
        catalog = api.portal.get_tool("portal_catalog")
        brains = catalog.searchResults(object_provides=taxonomy.getGeneratedName())
        # On parcours tous les brains pour lesquels la taxonomie est utilisée
        for brain in brains:
            index_key = f"taxonomy_{taxonomy.getShortName()}"
            if hasattr(brain, index_key):
                set2 = set(getattr(brain, index_key))
                if result & set2:
                    obj = brain.getObject()
                    url = obj.absolute_url()
                    title = obj.Title()
                    return json.dumps(
                        {
                            "status": "error",
                            "message": f'<h2>Impossible de supprimer ce terme.</h2><p>Ce terme est au moins utilisé ici : <a href="{url}" target="_blank" title="{title}">{url}</a></p>',
                        }
                    )
        return json.dumps({"status": "success", "message": "Suppression autorisée"})



# -*- coding: utf-8 -*-

from plone import api
from plone.app.z3cform.widget import AjaxSelectWidget
from z3c.form.interfaces import IFieldWidget
from z3c.form.widget import FieldWidget
from zope.interface import implementer
from zope.i18n import translate


class TranslatedAjaxSelectWidget(AjaxSelectWidget):
    """Translated Ajax select widget for z3c.form."""

    def _ajaxselect_options(self):
        options = super(TranslatedAjaxSelectWidget, self)._ajaxselect_options()
        current_lang = api.portal.get_current_language()[:2]
        translated_initial_values = {}
        for token, value in list(options.get("initialValues", {}).items()):
            translated_initial_values[token] = translate(
                value, domain="imio.smartweb", target_language=current_lang
            )
        if translated_initial_values:
            options["initialValues"] = translated_initial_values
        return options


@implementer(IFieldWidget)
def TranslatedAjaxSelectFieldWidget(field, request, extra=None):
    if extra is not None:
        request = extra
    return FieldWidget(field, TranslatedAjaxSelectWidget(request))



# -*- coding: utf-8 -*-

from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing.interfaces import SITE_OWNER_NAME
from plone.app.testing.interfaces import SITE_OWNER_PASSWORD
from plone.testing.z2 import Browser
from transaction import commit

import json
import unittest


class TestTaxonomy(unittest.TestCase):
    layer = IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.browser = Browser(self.layer["app"])
        self.browser.handleErrors = False
        self.browser.addHeader(
            "Authorization",
            "Basic {0}:{1}".format(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        commit()
        self.page = api.content.create(
            container=self.portal,
            type="Document",
            title="Test page",
        )

    def test_removing_unused_taxonomy_term(self):
        body_data = {
            "languages": ["fr", "en", "nl", "ru", "da", "de"],
            "tree": {
                "key": "0",
                "title": "Test vocabulary",
                "subnodes": [
                    {
                        "key": "1",
                        "translations": {
                            "da": "Informationsvidenskab",
                            "de": "Informatik",
                            "en": "Information Science",
                            "ru": "Информатику",
                        },
                        "subnodes": [
                            {
                                "key": "3",
                                "translations": {"da": "Kronologi", "en": "Chronology"},
                                "subnodes": [],
                            }
                        ],
                    }
                ],
                "default_language": "da",
            },
            "taxonomy": "collective.taxonomy.test",
        }
        json_str = json.dumps(body_data)
        bytes_data = json_str.encode("utf-8")
        self.request.set("BODY", bytes_data)
        self.request.method = "POST"
        result = self.portal.restrictedTraverse("@@taxonomy-import")()
        self.assertEqual(
            result,
            '{"status": "info", "message": "Your taxonomy has been saved with success."}',
        )



# -*- coding: utf-8 -*-

from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING
from imio.smartweb.common.testing import ImioSmartwebCommonTestCase
from imio.smartweb.common.viewlets.colophon import ColophonViewlet
from imio.smartweb.common.viewlets.skip_to_content import SkipToContentViewlet
from plone import api
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles


class TestViewlets(ImioSmartwebCommonTestCase):
    layer = IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="Folder",
            title="Folder",
            id="folder",
        )

    def test_colophon(self):
        logout()
        viewlet = ColophonViewlet(self.portal, self.request, None)
        viewlet.update()
        html = viewlet.render()
        self.assertIn("Agent connection", html)

        login(self.portal, "test")
        viewlet.update()
        html = viewlet.render()
        self.assertNotIn("Agent connection", html)

    def test_skip_to_content(self):
        logout()
        url = self.folder.absolute_url() + "/some_view"
        self.request["ACTUAL_URL"] = url
        self.request["QUERY_STRING"] = "foo=bar"
        viewlet = SkipToContentViewlet(self.folder, self.request, None)
        viewlet.update()
        html = viewlet.render()
        self.assertIn(
            'href="http://nohost/plone/folder/some_view?foo=bar#content"', html
        )



# -*- coding: utf-8 -*-

from imio.smartweb.common.utils import activate_sending_data_to_odwb_for_staging
from imio.smartweb.common.utils import is_staging_or_local
from plone import api
from plone.restapi.services import Service
from zope.interface import implementer
from zope.interface import Interface

import logging
import requests

logger = logging.getLogger("imio.smartweb.common")


class IOdwbService(Interface):

    def available():
        """ """

    def reply():
        """ """

    def remove():
        """ """


@implementer(IOdwbService)
class OdwbService(Service):

    def available(self):
        staging_or_local = is_staging_or_local()
        if staging_or_local:
            if activate_sending_data_to_odwb_for_staging() is True:
                return True
            logger.info(
                "Don't send odwb data when we are in staging or local environment"
            )
        return not staging_or_local


class OdwbBaseEndpointGet(OdwbService):

    def __init__(self, context, request, odwb_imio_service, odbw_pushkey):
        self.odwb_api_push_url = "https://www.odwb.be/api/push/1.0"
        self.odwb_imio_service = odwb_imio_service
        self.odwb_pushkey = api.portal.get_registry_record(odbw_pushkey)

        self.context = context
        self.request = request
        self.__datas__ = []

    def odwb_query(self, url, payload, headers=None):
        if headers is None:
            headers = {"Content-Type": "application/json"}
        try:
            response = requests.post(url, headers=headers, data=payload)
            return response.text
        except requests.exceptions.ConnectionError as e:
            logger.error("ODWB : Connection error: %s", e)
            return "ODWB : Connection error occurred"
        except requests.exceptions.Timeout as e:
            logger.error("ODWB : Request timed out: %s", e)
            return "ODWB : Request timed out"
        except requests.exceptions.HTTPError as e:
            logger.error("ODWB : HTTP error occurred: %s", e)
            return "ODWB : HTTP error occurred"
        except Exception as e:
            logger.error("ODWB : An unexpected error occurred: %s", e)
            return "ODWB : Unexpected error occurred"



# -*- coding: utf-8 -*-

from DateTime import DateTime
from freezegun import freeze_time
from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_INTEGRATION_TESTING
from plone import api
from plone.app.contenttypes.interfaces import IFile
from plone.app.contenttypes.interfaces import IImage
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.namedfile.file import NamedBlobFile
from plone.namedfile.file import NamedBlobImage

import unittest


class TestDescription(unittest.TestCase):
    layer = IMIO_SMARTWEB_COMMON_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    @freeze_time("2023-12-14 9:00:00")
    def test_added_content_image(self):
        # content types for which effective date is equal to created date (thank to subrscriber)
        content_types = ["File", "Image"]
        frozen_time = DateTime()

        for ct in content_types:
            obj = api.content.create(
                container=self.portal,
                type=ct,
                title=f"My {ct}",
            )
            if IFile.providedBy(obj):
                obj.file = NamedBlobFile(data="file data", filename="file.txt")

            elif IImage.providedBy(obj):
                obj.image = NamedBlobImage(data="file data", filename="file.txt")

            self.assertEqual(obj.effective(), frozen_time)
            # subscriber added_content fix effective as created date for Image content type
            self.assertEqual(obj.effective(), obj.created())



# -*- coding: utf-8 -*-

from more_itertools import chunked
from zope.globalrequest import getRequest

import hashlib
import json
import logging
import requests

logger = logging.getLogger("imio.smartweb.core")


def get_restapi_query_lang(query=None):
    if query is None:
        request = getRequest()
        query = request.form
    langs = [
        key.split("translated_in_")[1]
        for key in query
        if key.startswith("translated_in_")
    ]
    if not langs or len(langs) > 1:
        # we don't know how to handle multiple languages query
        return "fr"
    return langs[0]


def batch_results(iterable, batch_size):
    return list(chunked(iterable, batch_size, strict=False))


def get_json(url, auth=None, timeout=5):
    headers = {"Accept": "application/json"}
    if auth is not None:
        headers["Authorization"] = auth
    try:
        response = requests.get(url, headers=headers, timeout=timeout)
    except requests.exceptions.Timeout:
        logger.warning(f"Timeout raised for requests : {url}")
        return None
    except Exception:
        return None
    if response.status_code != 200:
        return None
    return json.loads(response.text)


def hash_md5(text):
    return hashlib.md5(text.encode()).hexdigest()



# -*- coding: utf-8 -*-

from imio.smartweb.common.browser.cropping import SmartwebCroppingImageScalingFactory
from imio.smartweb.common.interfaces import ICropping
from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING
from plone import api
from plone.app.imagecropping import PAI_STORAGE_KEY
from plone.app.imagecropping.events import CroppingInfoChangedEvent
from plone.app.imagecropping.events import CroppingInfoRemovedEvent
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.namedfile.file import NamedBlobImage
from plone.testing.z2 import Browser
from zope.annotation.interfaces import IAnnotations
from zope.component import getMultiAdapter
from zope.event import notify

import os
import time
import transaction
import unittest


class TestCropping(unittest.TestCase):
    layer = IMIO_SMARTWEB_COMMON_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="Folder",
            title="Folder",
            id="folder",
        )

    def test_cropping_adapter(self):
        adapter = ICropping(self.folder, alternate=None)
        self.assertIsNotNone(adapter)
        self.assertNotIn("banner", adapter.get_scales("image", self.request))

        image = api.content.create(
            container=self.folder,
            type="Image",
            title="My image",
            id="my-image",
        )
        adapter = ICropping(image, alternate=None)
        self.assertIsNotNone(adapter)
        self.assertEqual([], adapter.get_scales("image", self.request))

    def test_cropping_factory(self):
        test_image = os.path.join(
            os.path.dirname(__file__), "resources/image_1800x700.png"
        )
        with open(test_image, "rb") as fd:
            self.folder.image = NamedBlobImage(data=fd.read(), filename=test_image)

        view = self.folder.restrictedTraverse("@@crop-image")
        view._crop(fieldname="image", scale="portrait_affiche", box=(1, 1, 200, 200))

        factory = SmartwebCroppingImageScalingFactory(self.folder)

        factory("image", "scale", 430, 608, "portrait_vignette")
        # same box as "affiche" scale
        self.assertEqual(factory.box, (1, 1, 200, 200))

        factory("image", "scale", 100, 100, "preview")
        # uncropped scale
        self.assertIsNone(factory.box)

        factory("image", "scale", 100, 100)
        self.assertIsNone(factory.box)

        api.content.transition(self.folder, "publish")
        transaction.commit()
        browser = Browser(self.layer["app"])
        browser.open(f"{self.folder.absolute_url()}/@@images/image/portrait_liste")
        self.assertEqual("image/png", browser.headers["content-type"])
        # only one crop is saved (the big one)
        annotation = IAnnotations(self.folder).get(PAI_STORAGE_KEY)
        self.assertEqual(annotation, {"image_portrait_affiche": (1, 1, 200, 200)})

    def test_cropping_view(self):
        cropping_view = getMultiAdapter(
            (self.folder, self.request), name="croppingeditor"
        )
        self.assertEqual(len(list(cropping_view._scales("image"))), 14)
        size = [100, 100]
        scale_info = cropping_view._scale_info("image", "scale", size, size)
        self.assertEqual(scale_info["title"], "Scale")
        scale_info = cropping_view._scale_info("image", "portrait_scale", size, size)
        self.assertEqual(scale_info["title"], "Portrait")
        scale_info = cropping_view._scale_info("image", "paysage_scale", size, size)
        self.assertEqual(scale_info["title"], "Paysage")
        scale_info = cropping_view._scale_info("image", "carre_scale", size, size)
        self.assertEqual(scale_info["title"], "Carre")
        scale_info = cropping_view._scale_info("image", "unexpected_scale", size, size)
        self.assertEqual(scale_info["title"], "Unexpected_Scale")

    def test_modified_cropping(self):
        brain = api.content.find(UID=self.folder.UID())[0]
        dt_before = brain.ModificationDate
        time.sleep(1)
        notify(CroppingInfoChangedEvent(self.folder))
        brain = api.content.find(UID=self.folder.UID())[0]
        dt_after = brain.ModificationDate
        self.assertNotEqual(dt_before, dt_after)

        brain = api.content.find(UID=self.folder.UID())[0]
        dt_before = brain.ModificationDate
        time.sleep(1)
        notify(CroppingInfoRemovedEvent(self.folder))
        brain = api.content.find(UID=self.folder.UID())[0]
        dt_after = brain.ModificationDate
        self.assertNotEqual(dt_before, dt_after)



# -*- coding: utf-8 -*-

from plone.app.layout.viewlets import common
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getMultiAdapter


class SkipToContentViewlet(common.ViewletBase):
    index = ViewPageTemplateFile("skip_to_content.pt")

    def update(self):
        context_state = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )
        self.current_page_url = context_state.current_page_url



# -*- coding: utf-8 -*-

from imio.smartweb.common.testing import IMIO_SMARTWEB_COMMON_INTEGRATION_TESTING
from imio.smartweb.common.widgets.select import TranslatedAjaxSelectWidget
from plone.api import portal as portal_api
from zope.publisher.browser import TestRequest

import mock
import unittest


class TestVocabulary(unittest.TestCase):
    layer = IMIO_SMARTWEB_COMMON_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = TestRequest()

    def test_translated_ajax_select(self):
        portal_api.get_current_language = mock.Mock(return_value="fr")
        widget = TranslatedAjaxSelectWidget(self.request)
        widget.update()
        self.assertEqual(
            {
                "name": None,
                "value": "",
                "pattern": "select2",
                "pattern_options": {"separator": ";"},
            },
            widget._base_args(),
        )

        widget.vocabulary = "imio.smartweb.vocabulary.Topics"
        self.assertEqual(
            widget._base_args(),
            {
                "name": None,
                "value": "",
                "pattern": "select2",
                "pattern_options": {
                    "vocabularyUrl": "http://nohost/plone/@@getVocabulary?name=imio.smartweb.vocabulary.Topics",
                    "separator": ";",
                },
            },
        )
        widget.value = "entertainment"
        self.assertEqual(
            widget._base_args(),
            {
                "name": None,
                "value": "entertainment",
                "pattern": "select2",
                "pattern_options": {
                    "vocabularyUrl": "http://nohost/plone/@@getVocabulary?name=imio.smartweb.vocabulary.Topics",
                    "initialValues": {
                        "entertainment": "Activités et divertissement",
                    },
                    "separator": ";",
                },
            },
        )



# -*- coding: utf-8 -*-

from plone import api
from plone.app.content.browser.vocabulary import VocabularyView
from plone.app.content.utils import json_dumps
from plone.app.content.utils import json_loads
from zope.i18n import translate


class TranslatedVocabularyView(VocabularyView):

    def __call__(self):
        """
        Translate texts in JSON response
        `@getVocabulary` does not do it and returns English values
        """
        result = super(TranslatedVocabularyView, self).__call__()
        json = json_loads(result)
        if int(json.get("total", 0)) == 0:
            return result
        lang = api.portal.get_current_language()[:2]
        for item in json["results"]:
            if "text" in item:
                item["text"] = translate(
                    item["text"], domain="imio.smartweb", target_language=lang
                )
        return json_dumps(json)
